@@ -5,7 +5,7 @@
  *	Lu-chuan Kung and Kang-pen Chen.
  *	All rights reserved.
  *
- * Copyright (c) 2004, 2005, 2006, 2008
+ * Copyright (c) 2004, 2005, 2006, 2008, 2011
  *	libchewing Core Team. See ChangeLog for details.
  *
  * See the file "COPYING" for information on usage and redistribution
@@ -26,22 +26,25 @@
 #define HALFSHAPE_MODE 0
 
 /* specified to Chewing API */
-#ifdef WIN32
-#define CHEWING_DLL_IMPORT __declspec(dllimport)
+#if defined(_WIN32) || defined(_WIN64) || defined(_WIN32_WCE)
+#   define CHEWING_DLL_IMPORT __declspec(dllimport)
 #   define CHEWING_DLL_EXPORT __declspec(dllexport)
 #   ifdef CHEWINGDLL_EXPORTS
 #      define CHEWING_API CHEWING_DLL_EXPORT
 #      define CHEWING_PRIVATE
-#   else
+#   elif CHEWINGDLL_IMPORTS
 #      define CHEWING_API CHEWING_DLL_IMPORT
 #      define CHEWING_PRIVATE
+#   else
+#      define CHEWING_API
+#      define CHEWING_PRIVATE
 #   endif
-#elif (__GNUC__ > 3) && defined(__ELF__)
+#elif (__GNUC__ > 3) && (defined(__ELF__) || defined(__PIC__))
 #   define CHEWING_API __attribute__((__visibility__("default")))
 #   define CHEWING_PRIVATE __attribute__((__visibility__("hidden")))
 #else
-#  define CHEWING_API
-#  define CHEWING_PRIVATE
+#   define CHEWING_API
+#   define CHEWING_PRIVATE
 #endif
 
 #ifndef UNUSED
@@ -52,6 +55,7 @@
 #endif
 #endif
 
+#define MIN_SELKEY 1
 #define MAX_SELKEY 10
 
 /**
@@ -67,11 +71,7 @@ typedef struct {
 	int bAutoShiftCur;
 	int bEasySymbolInput;
 	int bPhraseChoiceRearward;
-	/** @brief
-            HSU_SELKEY_TYPE1 = asdfjkl789,
-            HSU_SELKEY_TYPE2 = asdfzxcv89.
-         */
-	int hsuSelKeyType;
+	int hsuSelKeyType; // Deprecated.
 } ChewingConfigData;
 
 typedef struct {
@@ -81,8 +81,6 @@ typedef struct {
 	/*@}*/
 } IntervalType;
 
-typedef unsigned short uint16;
-
 /** @brief context handle used for Chewing IM APIs
  */
 typedef struct _ChewingContext ChewingContext;
