@@ -12,6 +12,7 @@
 #define _CHEWING_CHAR_PRIVATE_H
 
 #include "global.h"
+#include "chewing-private.h"
 
 #ifndef SEEK_SET
 #define SEEK_SET 0
@@ -21,8 +22,9 @@ typedef struct {
 	char word[ 7 ];
 } Word;
 
-int GetCharFirst( Word *, uint16 );
-int GetCharNext ( Word * );
-int InitChar( const char * );
+int GetCharFirst( ChewingData *, Word *, uint16_t );
+int GetCharNext ( ChewingData *, Word * );
+int InitChar( ChewingData *pgdata, const char * prefix );
+void TerminateChar( ChewingData *pgdata );
 
 #endif
