@@ -44,7 +44,7 @@ int plat_mmap_is_valid( plat_mmap *handle )
 }
 
 /* map a file into system memory, return size of the mapped file */
-size_t plat_mmap_create( plat_mmap *handle, char *file, int fileAccessAttr )
+size_t plat_mmap_create( plat_mmap *handle, const char *file, int fileAccessAttr )
 {
 	size_t sizet;
 
