@@ -46,10 +46,12 @@ __forceinline void DEBUG_OUT( char* str, ... ){ }
 	(type *) calloc( size, sizeof( type ) )
 
 #define STATIC_ASSERT( exp, name ) typedef int STATIC_ASSERT_##name [ ( exp ) ? 1 : -1 ];
+#ifdef __GNUC__
+#define ARRAY_SIZE( array ) ( sizeof(array) / sizeof(((typeof(array)){})[0]) )
+#else
+#define ARRAY_SIZE( array ) ( sizeof(array) / sizeof(array[0] ) )
+#endif
 
 typedef int (*CompFuncType)( const void *, const void * );
 
-#define TerminateServicesNUM 10
-typedef void (*callback_t)();
-extern int addTerminateService( callback_t );
 #endif
