@@ -5,7 +5,7 @@
  *	Lu-chuan Kung and Kang-pen Chen.
  *	All rights reserved.
  *
- * Copyright (c) 2004, 2005, 2006, 2008
+ * Copyright (c) 2004, 2005, 2006, 2008, 2011
  *	libchewing Core Team. See ChangeLog for details.
  *
  * See the file "COPYING" for information on usage and redistribution
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "chewing-private.h"
 #include "chewing-utf8-util.h"
 #include "chewing-definition.h"
 #include "userphrase-private.h"
@@ -55,14 +56,6 @@ typedef struct {
 	int nPhListLen;
 } TreeDataType;
 
-#ifdef USE_BINARY_DATA
-TreeType *tree;
-static unsigned int tree_size = 0;
-static plat_mmap tree_mmap;
-#else
-TreeType tree[ TREE_SIZE ];
-#endif
-
 static int IsContain( IntervalType in1, IntervalType in2 )
 {
 	return ( in1.from <= in2.from && in1.to >= in2.to );
@@ -95,51 +88,73 @@ static int GetIntersection( IntervalType in1, IntervalType in2, IntervalType *in
 }
 #endif
 
-static void TerminateTree()
+void TerminateTree( ChewingData *pgdata )
 {
 #ifdef USE_BINARY_DATA
-	plat_mmap_close( &tree_mmap );
+		pgdata->static_data.tree = NULL;
+		plat_mmap_close( &pgdata->static_data.tree_mmap );
+#else
+		free( pgdata->static_data.tree );
+		pgdata->static_data.tree = NULL;
 #endif
 }
 
-void InitTree( const char *prefix )
+
+int InitTree( ChewingData *pgdata, const char * prefix )
 {
+#ifdef USE_BINARY_DATA
 	char filename[ PATH_MAX ];
+	size_t len;
+	size_t offset;
 
-#ifdef USE_BINARY_DATA
-	size_t offset = 0;
-	size_t csize;
+	len = snprintf( filename, sizeof( filename ), "%s" PLAT_SEPARATOR "%s", prefix, PHONE_TREE_FILE );
+	if ( len + 1 > sizeof( filename ) )
+		return -1;
+
+	plat_mmap_set_invalid( &pgdata->static_data.tree_mmap );
+	pgdata->static_data.tree_size = plat_mmap_create( &pgdata->static_data.tree_mmap, filename, FLAG_ATTRIBUTE_READ );
+	if ( pgdata->static_data.tree_size <= 0 )
+		return -1;
+
+	offset = 0;
+	pgdata->static_data.tree = (TreeType *) plat_mmap_set_view( &pgdata->static_data.tree_mmap, &offset, &pgdata->static_data.tree_size );
+	if ( !pgdata->static_data.tree )
+		return -1;
+
+	return 0;
 #else
-	FILE *infile;
+	char filename[ PATH_MAX ];
+	int len;
+	FILE *infile = NULL;
 	int i;
-#endif
 
-	sprintf( filename, "%s" PLAT_SEPARATOR "%s", prefix, PHONE_TREE_FILE );
-#ifdef USE_BINARY_DATA
-	plat_mmap_set_invalid( &tree_mmap );
-	tree_size = plat_mmap_create( &tree_mmap, filename, FLAG_ATTRIBUTE_READ );
-	assert( plat_mmap_is_valid( &tree_mmap ) );
-	if ( tree_size <= 0 )
-		return;
+	len = snprintf( filename, sizeof( filename ), "%s" PLAT_SEPARATOR "%s", prefix, PHONE_TREE_FILE );
+	if ( len + 1 > sizeof( filename ) )
+		return -1;
 
-	csize = tree_size;
-	tree = (TreeType *) plat_mmap_set_view( &tree_mmap, &offset, &csize );
-	assert( tree );
-#else
 	infile = fopen( filename, "r" );
-	assert( infile );
+	if ( !infile )
+		return -1;
+
+	pgdata->static_data.tree = ALC( TreeType, TREE_SIZE );
+	if ( !pgdata->static_data.tree ) {
+		fclose( infile );
+		return -1;
+	}
+
+	/* XXX: What happen if infile contains more than TREE_SIZE data? */
 	for ( i = 0; i < TREE_SIZE; i++ ) {
 		if ( fscanf( infile, "%hu%d%d%d",
-			&tree[ i ].phone_id,
-			&tree[ i ].phrase_id,
-			&tree[ i ].child_begin,
-			&tree[ i ].child_end ) != 4 )
+					&pgdata->static_data.tree[ i ].phone_id,
+					&pgdata->static_data.tree[ i ].phrase_id,
+					&pgdata->static_data.tree[ i ].child_begin,
+					&pgdata->static_data.tree[ i ].child_end ) != 4 )
 			break;
 	}
-	fclose( infile );
-#endif
 
-	addTerminateService( TerminateTree );
+	fclose( infile );
+	return 0;
+#endif
 }
 
 static int CheckBreakpoint( int from, int to, int bArrBrkpt[] )
@@ -152,7 +167,8 @@ static int CheckBreakpoint( int from, int to, int bArrBrkpt[] )
 }
 
 static int CheckUserChoose( 
-		uint16 *new_phoneSeq, int from , int to,
+		ChewingData *pgdata,
+		uint16_t *new_phoneSeq, int from , int to,
 		Phrase **pp_phr, 
 		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
 		IntervalType selectInterval[], int nSelect )
@@ -184,7 +200,7 @@ static int CheckUserChoose(
 	 * if there exist one phrase satisfied all selectStr then return 1, else return 0.
 	 * also store the phrase with highest freq
 	 */
-	pUserPhraseData = UserGetPhraseFirst( new_phoneSeq );
+	pUserPhraseData = UserGetPhraseFirst( pgdata, new_phoneSeq );
 	p_phr->freq = -1;
 	do {
 		for ( chno = 0; chno < nSelect; chno++ ) {
@@ -216,7 +232,7 @@ static int CheckUserChoose(
 				*pp_phr = p_phr;
 			}
 		}
-	} while ( ( pUserPhraseData = UserGetPhraseNext( new_phoneSeq ) ) != NULL );
+	} while ( ( pUserPhraseData = UserGetPhraseNext( pgdata, new_phoneSeq ) ) != NULL );
 
 	if ( p_phr->freq != -1 ) 
 		return 1;
@@ -229,6 +245,7 @@ static int CheckUserChoose(
  * phrase is said to satisfy a choose interval if 
  * their intersections are the same */
 static int CheckChoose(
+		ChewingData *pgdata,
 		int ph_id, int from, int to, Phrase **pp_phr, 
 		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
 		IntervalType selectInterval[], int nSelect )
@@ -243,7 +260,7 @@ static int CheckChoose(
 	*pp_phr = NULL;
 
 	/* if there exist one phrase satisfied all selectStr then return 1, else return 0. */
-	GetPhraseFirst( phrase, ph_id );
+	GetPhraseFirst( pgdata, phrase, ph_id );
 	do {
 		for ( chno = 0; chno < nSelect; chno++ ) {
 			c = selectInterval[ chno ];
@@ -269,7 +286,7 @@ static int CheckChoose(
 			*pp_phr = phrase;
 			return 1;
 		}
-	} while ( GetPhraseNext( phrase ) );
+	} while ( GetPhraseNext( pgdata, phrase ) );
 	free( phrase );
 	return 0;
 }
@@ -277,30 +294,31 @@ static int CheckChoose(
 /** @brief search for the phrases have the same pronunciation.*/
 /* if phoneSeq[a] ~ phoneSeq[b] is a phrase, then add an interval
  * from (a) to (b+1) */
-int TreeFindPhrase( int begin, int end, const uint16 *phoneSeq )
+int TreeFindPhrase( ChewingData *pgdata, int begin, int end, const uint16_t *phoneSeq )
 {
 	int child, tree_p, i;
 
 	tree_p = 0;
 	for ( i = begin; i <= end; i++ ) {
 		for ( 
-			child = tree[ tree_p ].child_begin;
-			child != -1 && child <= tree[ tree_p ].child_end;
+			child = pgdata->static_data.tree[ tree_p ].child_begin;
+			child != -1 && child <= pgdata->static_data.tree[ tree_p ].child_end;
 			child++ ) {
 
-			assert(0 <= child && child * sizeof(TreeType) < tree_size);
-
-			if ( tree[ child ].phone_id == phoneSeq[ i ] )
+#ifdef USE_BINARY_DATA
+			assert(0 <= child && child * sizeof(TreeType) < pgdata->static_data.tree_size);
+#endif
+			if ( pgdata->static_data.tree[ child ].phone_id == phoneSeq[ i ] )
 				break;
 		}
 		/* if not found any word then fail. */
-		if ( child == -1 || child > tree[ tree_p ].child_end )
+		if ( child == -1 || child > pgdata->static_data.tree[ tree_p ].child_end )
 			return -1;
 		else {
 			tree_p = child;
 		}
 	}
-	return tree[ tree_p ].phrase_id;
+	return pgdata->static_data.tree[ tree_p ].phrase_id;
 }
 
 static void AddInterval(
@@ -343,46 +361,43 @@ static void internal_release_Phrase( UsedPhraseMode mode, Phrase *pUser, Phrase
 	}
 }
 
-static void FindInterval(
-		uint16 *phoneSeq, int nPhoneSeq, 
-		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
-		IntervalType selectInterval[], int nSelect, 
-		int bArrBrkpt[], TreeDataType *ptd )
+static void FindInterval( ChewingData *pgdata, TreeDataType *ptd )
 {
 	int end, begin, pho_id;
 	Phrase *p_phrase, *puserphrase, *pdictphrase;
 	UsedPhraseMode i_used_phrase;
-	uint16 new_phoneSeq[ MAX_PHONE_SEQ_LEN ];
+	uint16_t new_phoneSeq[ MAX_PHONE_SEQ_LEN ];
 
-	for ( begin = 0; begin < nPhoneSeq; begin++ ) {
-		for ( end = begin; end < nPhoneSeq; end++ ) {
-			if ( ! CheckBreakpoint( begin, end + 1, bArrBrkpt ) )
+	for ( begin = 0; begin < pgdata->nPhoneSeq; begin++ ) {
+		for ( end = begin; end < pgdata->nPhoneSeq; end++ ) {
+			if ( ! CheckBreakpoint( begin, end + 1, pgdata->bArrBrkpt ) )
 				continue;
 
 			/* set new_phoneSeq */
 			memcpy( 
-				new_phoneSeq, 
-				&phoneSeq[ begin ], 
-				sizeof( uint16 ) * ( end - begin + 1 ) );
+				new_phoneSeq,
+				&pgdata->phoneSeq[ begin ],
+				sizeof( uint16_t ) * ( end - begin + 1 ) );
 			new_phoneSeq[ end - begin + 1 ] = 0;
 			puserphrase = pdictphrase = NULL;
 			i_used_phrase = USED_PHRASE_NONE;
 
 			/* check user phrase */
-			if ( UserGetPhraseFirst( new_phoneSeq ) &&
-					CheckUserChoose( new_phoneSeq, begin, end + 1, 
-					&p_phrase, selectStr, selectInterval, nSelect ) ) {
+			if ( UserGetPhraseFirst( pgdata, new_phoneSeq ) &&
+					CheckUserChoose( pgdata, new_phoneSeq, begin, end + 1,
+					&p_phrase, pgdata->selectStr, pgdata->selectInterval, pgdata->nSelect ) ) {
 				puserphrase = p_phrase;
 			}
 
 			/* check dict phrase */
-			pho_id = TreeFindPhrase( begin, end, phoneSeq );
+			pho_id = TreeFindPhrase( pgdata, begin, end, pgdata->phoneSeq );
 			if ( 
 				( pho_id != -1 ) && 
 				CheckChoose( 
+					pgdata,
 					pho_id, begin, end + 1, 
-					&p_phrase, selectStr, 
-					selectInterval, nSelect ) ) {
+					&p_phrase, pgdata->selectStr,
+					pgdata->selectInterval, pgdata->nSelect ) ) {
 				pdictphrase = p_phrase;
 			}
 
@@ -489,6 +504,13 @@ static int CompRecord( const RecordNode **pa, const RecordNode **pb )
 }
 
 
+/*
+ * Remove the interval containing in another interval.
+ *
+ * Example:
+ * 國民大會 has three interval: 國民, 大會, 國民大會. This function removes
+ * 國名, 大會 becasue 國民大會 contains 國民 and 大會.
+ */
 static void Discard1( TreeDataType *ptd )
 {
 	int a, b;
@@ -544,6 +566,17 @@ static void Discard1( TreeDataType *ptd )
 	ptd->nInterval = nInterval2;
 }
 
+/*
+ * Remove the interval that cannot connect to head or tail by other intervals.
+ *
+ * Example:
+ * The input string length is 5
+ * The available intervals are [1,1], [1,2], [2,3], [2,4], [5,5], [3,5].
+ *
+ * The possible connection from head to tail are [1,2][3,5], and
+ * [1,1][2,4][5,5]. Since [2,3] cannot connect to head or tail, it is removed
+ * by this function.
+ */
 static void Discard2( TreeDataType *ptd )
 {
 	int i, j;
@@ -578,24 +611,25 @@ static void Discard2( TreeDataType *ptd )
 	ptd->nInterval = nInterval2;
 }
 
-static void LoadChar( char *buf, int buf_len, uint16 phoneSeq[], int nPhoneSeq )
+static void LoadChar( ChewingData *pgdata, char *buf, int buf_len, uint16_t phoneSeq[], int nPhoneSeq )
 {
 	int i;
 	Word word;
 
 	memset(buf, 0, buf_len);
 	for ( i = 0; i < nPhoneSeq; i++ ) {
-		GetCharFirst( &word, phoneSeq[ i ] );
-		strncat(buf, word.word, buf_len);
+		GetCharFirst( pgdata, &word, phoneSeq[ i ] );
+		strncat(buf, word.word, buf_len - strlen(buf) - 1);
 	}
 	buf[ buf_len - 1 ] = '\0';
 }
 
 /* kpchen said, record is the index array of interval */
 static void OutputRecordStr(
+		ChewingData *pgdata,
 		char *out_buf, int out_buf_len,
 		int *record, int nRecord, 
-		uint16 phoneSeq[], int nPhoneSeq, 
+		uint16_t phoneSeq[], int nPhoneSeq,
 		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
 		IntervalType selectInterval[],
 		int nSelect, TreeDataType *ptd )
@@ -603,7 +637,7 @@ static void OutputRecordStr(
 	PhraseIntervalType inter;
 	int i;
 
-	LoadChar( out_buf, out_buf_len, phoneSeq, nPhoneSeq );
+	LoadChar( pgdata, out_buf, out_buf_len, phoneSeq, nPhoneSeq );
 	for ( i = 0; i < nRecord; i++ ) {
 		inter = ptd->interval[ record[ i ] ];
 		ueStrNCpy(
@@ -936,26 +970,20 @@ static RecordNode* NextCut( TreeDataType *tdt, PhrasingOutput *ppo )
 	return tdt->phList;
 }
 
-int Phrasing(
-		PhrasingOutput *ppo, uint16 phoneSeq[], int nPhoneSeq, 
-		char selectStr[][ MAX_PHONE_SEQ_LEN * MAX_UTF8_SIZE + 1 ], 
-		IntervalType selectInterval[], int nSelect, 
-		int bArrBrkpt[], int bUserArrCnnct[] ) 
+int Phrasing( ChewingData *pgdata )
 {
 	TreeDataType treeData;
 
 	InitPhrasing( &treeData );
 
-	FindInterval( 
-		phoneSeq, nPhoneSeq, selectStr, selectInterval, nSelect, 
-		bArrBrkpt, &treeData );
-	SetInfo( nPhoneSeq, &treeData );
+	FindInterval( pgdata, &treeData );
+	SetInfo( pgdata->nPhoneSeq, &treeData );
 	Discard1( &treeData );
 	Discard2( &treeData );
 	SaveList( &treeData );
-	CountMatchCnnct( &treeData, bUserArrCnnct, nPhoneSeq );
+	CountMatchCnnct( &treeData, pgdata->bUserArrCnnct, pgdata->nPhoneSeq );
 	SortListByScore( &treeData );
-	NextCut( &treeData, ppo );
+	NextCut( &treeData, &pgdata->phrOut );
 
 #ifdef ENABLE_DEBUG
 	ShowList( &treeData );
@@ -964,13 +992,14 @@ int Phrasing(
 
 	/* set phrasing output */
 	OutputRecordStr(
-		ppo->chiBuf, sizeof(ppo->chiBuf),
+		pgdata,
+		pgdata->phrOut.chiBuf, sizeof(pgdata->phrOut.chiBuf),
 		( treeData.phList )->arrIndex, 
 		( treeData.phList )->nInter, 
-		phoneSeq, 
-		nPhoneSeq, 
-		selectStr, selectInterval, nSelect, &treeData );
-	SaveDispInterval( ppo, &treeData );
+		pgdata->phoneSeq,
+		pgdata->nPhoneSeq,
+		pgdata->selectStr, pgdata->selectInterval, pgdata->nSelect, &treeData );
+	SaveDispInterval( &pgdata->phrOut, &treeData );
 
 	/* free "phrase" */
 	CleanUpMem( &treeData );
